class Nonces(object):
    AbstractTransactable = "fake-abstract-transactable-nonce"
    # NEXT_MAJOR_VERSION remove amex express checkout
    AmexExpressCheckoutCard = "fake-amex-express-checkout-nonce"
    # NEXT_MAJOR_VERSION - rename AndroidPay to GooglePay
    AndroidPayCard = "fake-android-pay-nonce"
    AndroidPayCardAmEx = "fake-android-pay-amex-nonce"
    AndroidPayCardDiscover = "fake-android-pay-discover-nonce"
    AndroidPayCardMasterCard = "fake-android-pay-mastercard-nonce"
    AndroidPayCardVisa = "fake-android-pay-visa-nonce"
    ApplePayAmEx = "fake-apple-pay-amex-nonce"
    ApplePayMasterCard = "fake-apple-pay-mastercard-nonce"
    ApplePayMpan = "fake-apple-pay-mpan-nonce"
    ApplePayVisa = "fake-apple-pay-visa-nonce"
    Consumed = "fake-consumed-nonce"
    Europe = "fake-europe-bank-account-nonce"
    GatewayRejectedFraud = "fake-gateway-rejected-fraud-nonce"
    GatewayRejectedRiskThreshold = "fake-gateway-rejected-risk-thresholds-nonce"
    LocalPayment = "fake-local-payment-method-nonce"
    LuhnInvalid = "fake-luhn-invalid-nonce"
    # NEXT_MAJOR_VERSION remove masterpass
    MasterpassAmEx = "fake-masterpass-amex-nonce"
    MasterpassDiscover = "fake-masterpass-discover-nonce"
    MasterpassMasterCard = "fake-masterpass-mastercard-nonce"
    MasterpassVisa = "fake-masterpass-visa-nonce"
    PayPalBillingAgreement = "fake-paypal-billing-agreement-nonce"
    # NEXT_MAJOR_VERSION - no longer supported in the Gateway remove this constant
    PayPalFuturePayment = "fake-paypal-future-nonce"
    PayPalFuturePaymentRefreshToken = "fake-paypal-future-refresh-token-nonce"
    PayPalOneTimePayment = "fake-paypal-one-time-nonce"
    ProcessorDeclinedAmEx = "fake-processor-declined-amex-nonce"
    ProcessorDeclinedDiscover = "fake-processor-declined-discover-nonce"
    ProcessorDeclinedMasterCard = "fake-processor-declined-mastercard-nonce"
    ProcessorDeclinedVisa = "fake-processor-declined-visa-nonce"
    ProcessorFailureJCB = "fake-processor-failure-jcb-nonce"
    SEPA = "fake-sepa-bank-account-nonce"
    MetaCheckoutCard = "fake-meta-checkout-card-nonce"
    MetaCheckoutToken = "fake-meta-checkout-token-nonce"
    VisaCheckoutAmEx = "fake-visa-checkout-amex-nonce"
    VisaCheckoutDiscover = "fake-visa-checkout-discover-nonce"
    VisaCheckoutMasterCard = "fake-visa-checkout-mastercard-nonce"
    VisaCheckoutVisa = "fake-visa-checkout-visa-nonce"
    # NEXT_MAJOR_VERSION remove SamsungPay nonces
    SamsungPayAmex = "tokensam_fake_american_express"
    SamsungPayDiscover = "tokensam_fake_american_express"
    SamsungPayMasterCard = "tokensam_fake_mastercard"
    SamsungPayVisa = "tokensam_fake_visa"
    SepaDirectDebit = "fake-sepa-direct-debit-nonce"
    ThreeDSecureTwoVisaErrorOnLookup = "fake-three-d-secure-two-visa-error-on-lookup-nonce"
    ThreeDSecureTwoVisaSuccessfulFrictionlessAuthentication = "fake-three-d-secure-two-visa-successful-frictionless-authentication-nonce"
    ThreeDSecureTwoVisaSuccessfulStepUpAuthentication = "fake-three-d-secure-two-visa-successful-step-up-authentication-nonce"
    ThreeDSecureTwoVisaTimeoutOnLookup = "fake-three-d-secure-two-visa-timeout-on-lookup-nonce"
    ThreeDSecureVisaAttemptsNonParticipating = "fake-three-d-secure-visa-attempts-non-participating-nonce"
    ThreeDSecureVisaAuthenticationUnavailable = "fake-three-d-secure-visa-authentication-unavailable-nonce"
    ThreeDSecureVisaBypassedAuthentication = "fake-three-d-secure-visa-bypassed-authentication-nonce"
    ThreeDSecureVisaFailedAuthentication = "fake-three-d-secure-visa-failed-authentication-nonce"
    ThreeDSecureVisaFailedSignature = "fake-three-d-secure-visa-failed-signature-nonce"
    ThreeDSecureVisaFullAuthentication = "fake-three-d-secure-visa-full-authentication-nonce"
    ThreeDSecureVisaLookupTimeout = "fake-three-d-secure-visa-lookup-timeout-nonce"
    ThreeDSecureVisaMPIAuthenticateError = "fake-three-d-secure-visa-mpi-authenticate-error-nonce"
    ThreeDSecureVisaMPILookupError = "fake-three-d-secure-visa-mpi-lookup-error-nonce"
    ThreeDSecureVisaNoteEnrolled = "fake-three-d-secure-visa-not-enrolled-nonce"
    ThreeDSecureVisaUnavailable = "fake-three-d-secure-visa-unavailable-nonce"
    Transactable = "fake-valid-nonce"
    TransactableAmEx = "fake-valid-amex-nonce"
    TransactableCommercial = "fake-valid-commercial-nonce"
    TransactableCountryOfIssuanceCAD = "fake-valid-country-of-issuance-cad-nonce"
    TransactableCountryOfIssuanceUSA = "fake-valid-country-of-issuance-usa-nonce"
    TransactableDebit = "fake-valid-debit-nonce"
    TransactableDinersClub = "fake-valid-dinersclub-nonce"
    TransactableDiscover = "fake-valid-discover-nonce"
    TransactableDurbinRegulated = "fake-valid-durbin-regulated-nonce"
    TransactableHealthcare = "fake-valid-healthcare-nonce"
    TransactableIssuingBankNetworkOnly = "fake-valid-issuing-bank-network-only-nonce"
    TransactableJCB = "fake-valid-jcb-nonce"
    TransactableMaestro = "fake-valid-maestro-nonce"
    TransactableMasterCard = "fake-valid-mastercard-nonce"
    TransactableNoIndicators = "fake-valid-no-indicators-nonce"
    TransactablePayroll = "fake-valid-payroll-nonce"
    TransactablePinlessDebitVisa = "fake-pinless-debit-visa-nonce"
    TransactablePrepaid = "fake-valid-prepaid-nonce"
    TransactableUnknownIndicators = "fake-valid-unknown-indicators-nonce"
    TransactableVisa = "fake-valid-visa-nonce"
    VenmoAccount = "fake-venmo-account-nonce"
    VenmoAccountTokenIssuanceError = "fake-token-issuance-error-venmo-account-nonce"
    VisaCheckoutAmEx = "fake-visa-checkout-amex-nonce"
    VisaCheckoutDiscover = "fake-visa-checkout-discover-nonce"
    VisaCheckoutMasterCard = "fake-visa-checkout-mastercard-nonce"
    VisaCheckoutVisa = "fake-visa-checkout-visa-nonce"
    UsBankAccount = "fake-us-bank-account-nonce"