
class PaymentInstrumentType():
    # NEXT_MAJOR_VERSION remove amex express checkout, masterpass, 
    # and SamsungPayCard. They have been deprecated
    AmexExpressCheckoutCard = "amex_express_checkout_card"
    AndroidPayCard = "android_pay_card"
    ApplePayCard = "apple_pay_card"
    CreditCard = "credit_card"
    EuropeBankAccount = "europe_bank_account"
    LocalPayment = "local_payment"
    MasterpassCard = "masterpass_card"
    MetaCheckoutCard = "meta_checkout_card"
    MetaCheckoutToken = "meta_checkout_token"
    PayPalAccount = "paypal_account"
    PayPalHere = "paypal_here"
    SamsungPayCard = "samsung_pay_card"
    SepaDirectDebitAccount = "sepa_debit_account"
    UsBankAccount = "us_bank_account"
    VenmoAccount = "venmo_account"
    VisaCheckoutCard = "visa_checkout_card"
